// ---
// this code was copied (and modified) from LiveHTTPHeaders Copyright(c) 2002-2003 Daniel Savard

const HEADER_SPY_SERVICE_NAME = "Header Spy Service";
const HEADER_SPY_SERVICE_CID = Components.ID("{f6480eac-e13a-494c-85ba-4de1817abbbd}");
const HEADER_SPY_SERVICE_CONTRACTID = "@headerspy.mozdev.org/service;1";

const HEADER_SPY_CATMAN_CONTRACTID = "@mozilla.org/categorymanager;1";

function header_spy_visitor(oHttp)
{
    // Keep the Http request object
    this.oHttp = oHttp;

    // headers pseudo-object
    this.request = "";
    this.requestHeaders = [];
    this.response = "";
    this.responseHeaders = [];

    // Initialize headers 
    this.visitRequest(this);
    this.visitResponse(this);
}

header_spy_visitor.prototype =
{
    oHttp : null,
    headers : null,

    getHttpResponseVersion: function() {
        var maj = new Object();
        var min = new Object();
        this.oHttp.QueryInterface(Components.interfaces.nsIHttpChannelInternal);
        this.oHttp.getResponseVersion(maj, min);
        return ("" + maj.value + "." + min.value);
    },

    getHttpRequestVersion: function() {
        var maj = new Object();
        var min = new Object();
        this.oHttp.QueryInterface(Components.interfaces.nsIHttpChannelInternal);
        this.oHttp.getRequestVersion(maj, min);
        return ("" + maj.value + "." + min.value);
    },

    visitPostHeaders: function(visitor) {
/*
        // Get the headers from postData stream if present
        try {
            // Must change HttpChannel to UploadChannel to be able to access post data
            this.oHttp.QueryInterface(Components.interfaces.nsIUploadChannel);
            // Get the post data stream
            if (this.oHttp.uploadStream) {
                try {
                    // Must check if there is headers in the stream
                    this.oHttp.uploadStream.QueryInterface(Components.interfaces.nsIMIMEInputStream);
                    // Must change to SeekableStream to be able to rewind the stream
                    this.oHttp.uploadStream.QueryInterface(Components.interfaces.nsISeekableStream);
                    this.oHttp.uploadStream.seek(0, 0);

                    // Need to create a seekable stream to be able to read the stream in javascript
                    var stream = new JS_ScriptableInputStream();
                    stream.init(this.oHttp.uploadStream);

                    // Now we can start to get the headers
                    var line = this.readLine(stream);
                    while (line) {
                        var tmp = line.split(/:\s?/);
                        visitor.visitHeader(tmp[0], tmp[1]);
                        line = this.readLine(stream);
                    }
                } catch(ex) {
                    // --
                } finally { // Need to close the stream after use
                    this.oHttp.uploadStream.close();
                    stream.close();
              }
          }
        } catch(e) {}
*/
    },

    visitHeader : function(name, value) {
        this.theaders[name] = value;
    },

    visitRequest : function() {
        try {
            var uri = String(this.oHttp.URI.asciiSpec);
            var ver = this.getHttpRequestVersion(false);

            this.request = this.oHttp.requestMethod + " " + uri + " HTTP/" + ver;

            this.theaders = new Array();
            this.oHttp.visitRequestHeaders(this);
            this.visitPostHeaders(this);
            this.requestHeaders = this.theaders;
        } catch(e) {}
    },
  
    visitResponse : function() {
        try {
            var ver = this.getHttpResponseVersion();
            this.response = "HTTP/" + ver + " " + this.oHttp.responseStatus + " " + this.oHttp.responseStatusText;
            
            this.theaders = new Array();
            this.oHttp.visitResponseHeaders(this);
            this.responseHeaders = this.theaders;
        } catch(e) {}
    },
}

// --------------------------------

var header_spy_service =
{
    QueryInterface: function(iid) {
        if (iid.equals(Components.interfaces.nsISupports)
                || iid.equals(Components.interfaces.nsISupportsWeakReference)
                || iid.equals(Components.interfaces.nsIWebProgressListener)
                || iid.equals(Components.interfaces.nsIHttpNotify)
                || iid.equals(Components.interfaces.nsIObserver)
                ) {
            return this;
        }
        throw Components.results.NS_NOINTERFACE;
    },

    // ---

    observe: function(aSubject, aTopic, aData) {
//dump("header_spy: header_spy_service.observe: " + aTopic + "\n");
        if (aTopic == 'app-startup') {
            this.observers = [];
            this.delays    = [];

            var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
            observerService.addObserver(this, "http-on-modify-request", false);
        } else if (aTopic == 'http-on-modify-request') {
            aSubject.QueryInterface(Components.interfaces.nsIHttpChannel);
            this.onModifyRequest(aSubject);
//        } else if (aTopic == 'http-on-examine-response') {
//            aSubject.QueryInterface(Components.interfaces.nsIHttpChannel);
//            this.onExamineResponse(aSubject);
        }
    },

    addObserver : function (observer) {
        this.observers[observer] = observer;
    },

    removeObserver : function (observer) {
        delete this.observers[observer];
    },

    // ---
    
    onStateChange: function(aWebProgress, aRequest, aStateFlags, aStatus) {
//dump("header_spy: header_spy_service.onStateChange\n");
        if ((aStateFlags & Components.interfaces.nsIWebProgressListener.STATE_START)
                && (aStateFlags & Components.interfaces.nsIWebProgressListener.STATE_IS_DOCUMENT)
                ) {
//dump("header_spy: header_spy_service.onStateChange - 0-1\n");
            this.get_headers(aWebProgress, aRequest, false);
//dump("header_spy: header_spy_service.onStateChange - 0-2\n");
        } else if ((aStateFlags & Components.interfaces.nsIWebProgressListener.STATE_STOP)
                && (aStateFlags & Components.interfaces.nsIWebProgressListener.STATE_IS_DOCUMENT)
                ) {
//dump("header_spy: header_spy_service.onStateChange - 1-1\n");
            this.get_headers(aWebProgress, aRequest, true);
//dump("header_spy: header_spy_service.onStateChange - 1-2\n");
            //header_spy.rebuild(aWebProgress);
            try { // already removed?
                aWebProgress.removeProgressListener(this);
            } catch(e) {}
        }
    },
    onProgressChange: function (aWebProgress, aRequest, aCurSelfProgress, aMaxSelfProgress, aCurTotalProgress, aMaxTotalProgress) {
    },
    onLocationChange: function(aWebProgress, aRequest, aLocation) {
    },
    onStatusChange: function(aWebProgress, aRequest, aStatus, aMessage) {
    },
    onSecurityChange: function(aWebProgress, aRequest, aState) {
    },
    //onLinkIconAvailable: function(a) {
    //},

    // ---

    onModifyRequest: function(oHttp) {
        try {
            oHttp.QueryInterface(Components.interfaces.nsIRequest);

            // We only need to register a listener if this is a document uri as all embeded object
            // are checked by the same listener (not true for frames but frames are document uri...)
            if ((oHttp.loadFlags & Components.interfaces.nsIChannel.LOAD_DOCUMENT_URI)
                    && oHttp.loadGroup
                    && oHttp.loadGroup.groupObserver
                    ) {
                var go = oHttp.loadGroup.groupObserver;
                go.QueryInterface(Components.interfaces.nsIWebProgress);
                go.addProgressListener(this, Components.interfaces.nsIWebProgress.NOTIFY_STATE_DOCUMENT);
            }
        } catch(e) {}
    },
    onExamineResponse: function(oHttp) {
    },

    // ---

    init: function() {
    },
  
    destroy: function() {
    },
  
    get_headers: function(aWebProgress, aRequest, do_overwrite) {
//dump("header_spy: header_spy_service.get_headers - 0-1\n");
        if (true//!aWebProgress.isLoadingDocument
                //&& ('document' in aWebProgress)
                && ('DOMWindow' in aWebProgress)
                //&& ('document' in aWebProgress.DOMWindow)
                //&& !('header_spy_headers' in aWebProgress.document)
                //&& !('header_spy_headers' in aWebProgress.DOMWindow)
                && (do_overwrite || !('header_spy_headers' in aWebProgress.DOMWindow))
                ) {
//dump("header_spy: header_spy_service.get_headers - 0-2\n");
            try {
                var scheme = aRequest.QueryInterface(Components.interfaces.nsIChannel).URI.scheme;
                if (scheme == 'http' || scheme == 'https') {
//dump("header_spy: header_spy_service.get_headers - 0-3\n");
                    try {
                        aRequest.QueryInterface(Components.interfaces.nsIHttpChannel);
                    } catch(ex) {
                        aRequest.QueryInterface(Components.interfaces.nsIMultiPartChannel);
                        aRequest.baseChannel.QueryInterface(Components.interfaces.nsIHttpChannel);
                    }

//dump("header_spy: header_spy_service.get_headers - 0-4\n");
                    //aWebProgress.document.header_spy_headers = new header_spy_visitor(aRequest);
                    aWebProgress.DOMWindow.header_spy_headers = new header_spy_visitor(aRequest);
                    //aWebProgress.DOMWindow.document.header_spy_headers = new header_spy_visitor(aRequest);
//dump("header_spy: header_spy_service.get_headers - 0-5\n");
                }
            } catch(e) {}
        }
    },

};

// --------------------------------

/* Module (for XPCOM registration) */
var header_spy_service_module = {
    registerSelf: function(compMgr, fileSpec, location, type) {
        var compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(HEADER_SPY_SERVICE_CID, 
            HEADER_SPY_SERVICE_NAME, HEADER_SPY_SERVICE_CONTRACTID, fileSpec, location, type);

        // "app-startup" - General event for application startup. You must register in the category manager to see this topic.
        var catman = Components.classes[HEADER_SPY_CATMAN_CONTRACTID].getService(Components.interfaces.nsICategoryManager);
        catman.addCategoryEntry("app-startup", "HeaderSpy", HEADER_SPY_SERVICE_CONTRACTID, true, true);

    },

    unregisterSelf: function(compMgr, fileSpec, location) {
        // Remove the auto-startup
        var catman = Components.classes[HEADER_SPY_CATMAN_CONTRACTID].getService(Components.interfaces.nsICategoryManager);
        catman.deleteCategoryEntry("app-startup", HEADER_SPY_SERVICE_CONTRACTID, true);

        var compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.unregisterFactoryLocation(HEADER_SPY_SERVICE_CID, fileSpec);
    },

    getClassObject: function(compMgr, cid, iid) {
        if (!cid.equals(HEADER_SPY_SERVICE_CID))
            throw Components.results.NS_ERROR_NO_INTERFACE;

        if (!iid.equals(Components.interfaces.nsIFactory))
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

        return header_spy_service_factory;
    },

    canUnload: function(compMgr) {
        return true;
    },
};

/* Class Factory */
var header_spy_service_factory = {
    createInstance: function(outer, iid) {
        if (outer != null) {
            throw Components.results.NS_ERROR_NO_AGGREGATION;
        }

        if (iid.equals(Components.interfaces.nsISupports)
                || iid.equals(Components.interfaces.nsIObserver)
                ) {
            return header_spy_service;
            //return (new header_spy_service()).QueryInterface(iid);
        }
        throw Components.results.NS_ERROR_INVALID_ARG;
    },
}

/* module initialization */
function NSGetModule(comMgr, fileSpec) {
    return header_spy_service_module;
}

// --------------------------------

